"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.memoize = memoize;
exports.memoizeByKey = memoizeByKey;
exports.eqCheck = void 0;

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _wrapNativeSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/wrapNativeSuper"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _constants = require("./constants");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var defaultEqualityCheck = function defaultEqualityCheck(a, b) {
  return a === b;
};

var eqCheck = defaultEqualityCheck;
exports.eqCheck = eqCheck;

var isOrmState = function isOrmState(arg) {
  return arg && (0, _typeof2["default"])(arg) === "object" && arg.hasOwnProperty(_constants.STATE_FLAG);
};

var argsAreEqual = function argsAreEqual(lastArgs, nextArgs, equalityCheck) {
  return nextArgs.every(function (arg, index) {
    return isOrmState(arg) && isOrmState(lastArgs[index]) || equalityCheck(arg, lastArgs[index]);
  });
};

var rowsAreEqual = function rowsAreEqual(ids, rowsA, rowsB) {
  return ids.every(function (id) {
    return rowsA[id] === rowsB[id];
  });
};

var accessedModelInstancesAreEqual = function accessedModelInstancesAreEqual(previous, ormState, orm) {
  var accessedInstances = previous.accessedInstances;
  return Object.entries(accessedInstances).every(function (_ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
        modelName = _ref2[0],
        instances = _ref2[1];

    // if the entire table has not been changed, we have nothing to do
    if (previous.ormState[modelName] === ormState[modelName]) {
      return true;
    }

    var _orm$getDatabase$desc = orm.getDatabase().describe(modelName),
        mapName = _orm$getDatabase$desc.mapName;

    var previousRows = previous.ormState[modelName][mapName];
    var rows = ormState[modelName][mapName];
    var accessedIds = Object.keys(instances);
    return rowsAreEqual(accessedIds, previousRows, rows);
  });
};

var accessedIndexesAreEqual = function accessedIndexesAreEqual(previous, ormState) {
  var accessedIndexes = previous.accessedIndexes;
  return Object.entries(accessedIndexes).every(function (_ref3) {
    var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
        modelName = _ref4[0],
        indexes = _ref4[1];

    return Object.entries(indexes).every(function (_ref5) {
      var _ref6 = (0, _slicedToArray2["default"])(_ref5, 2),
          column = _ref6[0],
          values = _ref6[1];

      return values.every(function (value) {
        return previous.ormState[modelName].indexes[column][value] === ormState[modelName].indexes[column][value];
      });
    });
  });
};

var fullTableScannedModelsAreEqual = function fullTableScannedModelsAreEqual(previous, ormState) {
  return previous.fullTableScannedModels.every(function (modelName) {
    return previous.ormState[modelName] === ormState[modelName];
  });
};

var MEMOIZATION_STATE_KEY = "@@_______REDUX_ORM_MEMOIZATION_STATE";
var MEMOIZATION_STATE = {
  /* Result of the previous function call */
  result: null,

  /**
   * Snapshot of the previous database.
   *
   * Lets us know how the tables looked like
   * during the previous function call.
   */
  ormState: null,

  /**
   * Names of models whose tables have been scanned completely
   * during previous function call (contains only model names)
   * Format example: ['Book']
   */
  fullTableScannedModels: [],

  /**
   * Map of which model instances have been accessed
   * during previous function call.
   * Contains only PKs of accessed instances.
   * Format example: { Book: { 1: true, 3: true } }
   */
  accessedInstances: {},

  /**
   * Map of which attribute indexes have been accessed
   * during previous function call.
   * Contains only attributes that were actually filtered on.
   * Author.withId(3).books would add 3 to the authorId index below.
   * Format example: { Book: { authorId: [1, 2], publisherId: [5] } }
   */
  accessedIndexes: {},

  /* Whether the selector has been called */
  calledBefore: false
};
/**
 * A memoizer to use with redux-orm
 * selectors. When the memoized function is first run,
 * the memoizer will remember the models that are accessed
 * during that function run.
 *
 * On subsequent runs, the memoizer will check if those
 * models' states have changed compared to the previous run.
 *
 * Memoization algorithm operates like this:
 *
 * 1. Has the selector been run before? If not, go to 6.
 *
 * 2. If the selector has other input selectors in addition to the
 *    ORM state selector, check their results for equality with the previous results.
 *    If they aren't equal, go to 6.
 *
 * 3. Some filter queries may have required scanning entire tables during the last run.
 *    If any of those tables have changed, go to 6.
 *
 * 4. Check which foreign key indexes the database has used to speed up queries
 *    during the last run. If any have changed, go to 6.
 *
 * 5. Check which Model's instances the selector has accessed during the last run.
 *    Check for equality with each of those states versus their states in the
 *    previous ORM state. If all of them are equal, return the previous result.
 *
 * 6. Run the selector. Check the Session object used by the selector for
 *    which Model's states were accessed, and merge them with the previously
 *    saved information about accessed models (if-else branching can change
 *    which models are accessed on different inputs). Save the ORM state and
 *    other arguments the selector was called with, overriding previously
 *    saved values. Save the selector result. Return the selector result.
 *
 * @private
 * @param  {Function} func - function to memoize
 * @param  {ORM} orm - a redux-orm ORM instance
 * @return {Function} `func` memoized.
 */

function memoize(func, orm) {
  var previous = _objectSpread(_objectSpread({}, MEMOIZATION_STATE), {}, {
    args: null
  });

  return function () {
    for (var _len = arguments.length, stateAndArgs = new Array(_len), _key = 0; _key < _len; _key++) {
      stateAndArgs[_key] = arguments[_key];
    }

    /**
     * The first argument to this function needs to be
     * the ORM's reducer state in the user's Redux store.
     */
    var ormState = stateAndArgs[0],
        args = stateAndArgs.slice(1);

    if (previous.calledBefore && argsAreEqual(previous.args, args, defaultEqualityCheck) && fullTableScannedModelsAreEqual(previous, ormState) && accessedIndexesAreEqual(previous, ormState) && accessedModelInstancesAreEqual(previous, ormState, orm)) {
      /**
       * None of this selector's dependencies have changed
       * since the last time that we called it.
       */
      return previous.result;
    }
    /**
     * Start a session so that the selector can access the database.
     * Make this session immutable. This way we can find out if
     * the operations that the selector performs are cacheable.
     */


    var session = orm.session(ormState);
    /* Replace all ORM state arguments by the session above */

    var argsWithSession = args.map(function (arg) {
      return isOrmState(arg) ? session : arg;
    });
    /* This is where we call the actual function */

    var result = func.apply(null, argsWithSession); // eslint-disable-line prefer-spread

    /**
     * The metadata for the previous call are no longer valid.
     * Update cached values.
     */

    previous.result = result;
    previous.args = args;
    /* Redux state slice for session.state */

    previous.ormState = ormState;
    /* Rows retrieved by resolved primary key */

    previous.accessedInstances = session.accessedModelInstances;
    /* Foreign key indexes that were used to speed up queries */

    previous.accessedIndexes = session.accessedIndexes;
    /* Tables that had to be scanned completely */

    previous.fullTableScannedModels = session.fullTableScannedModels;
    /* Whether the selector has been called */

    previous.calledBefore = true;
    return result;
  };
}

var PreviousCalls = /*#__PURE__*/function (_Map) {
  (0, _inheritsLoose2["default"])(PreviousCalls, _Map);

  function PreviousCalls() {
    return _Map.apply(this, arguments) || this;
  }

  var _proto = PreviousCalls.prototype;

  _proto.getFromCacheKeys = function getFromCacheKeys(cacheKeys) {
    var level = this;

    var _iterator = _createForOfIteratorHelper(cacheKeys),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var key = _step.value;

        if (level.has(key)) {
          level = level.get(key);
          continue;
        }

        var newLevel = new Map([[MEMOIZATION_STATE_KEY, _objectSpread({}, MEMOIZATION_STATE)]]);
        level.set(key, newLevel);
        level = newLevel;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return level;
  };

  _proto.getState = function getState(cacheKeys) {
    var call = this.getFromCacheKeys(cacheKeys);
    return call ? call.get(MEMOIZATION_STATE_KEY) : null;
  };

  return PreviousCalls;
}( /*#__PURE__*/(0, _wrapNativeSuper2["default"])(Map));

function memoizeByKey(func, orm, ignoreDependenciesCount) {
  var previousCalls = new PreviousCalls([[MEMOIZATION_STATE_KEY, _objectSpread({}, MEMOIZATION_STATE)]]);
  return function () {
    for (var _len2 = arguments.length, stateAndKeysAndArgs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      stateAndKeysAndArgs[_key2] = arguments[_key2];
    }

    /**
     * The first argument to this function needs to be
     * the ORM's reducer state in the user's Redux store.
     */
    var ormState = stateAndKeysAndArgs[0],
        cacheKeys = stateAndKeysAndArgs[1],
        args = stateAndKeysAndArgs.slice(2);
    /** Ignore the first `ignoreDependenciesCount` dependencies. */

    var argsToCompare = args.slice(Math.max(0, ignoreDependenciesCount));
    var previous = previousCalls.getState([cacheKeys]);

    if (previous.calledBefore && argsAreEqual(previous.argsToCompare, argsToCompare, defaultEqualityCheck) && fullTableScannedModelsAreEqual(previous, ormState) && accessedIndexesAreEqual(previous, ormState) && accessedModelInstancesAreEqual(previous, ormState, orm)) {
      /**
       * None of this selector's dependencies have changed
       * since the last time that we called it.
       */
      return previous.result;
    }
    /**
     * Start a session so that the selector can access the database.
     * Make this session immutable. This way we can find out if
     * the operations that the selector performs are cacheable.
     */


    var session = orm.session(ormState);
    /* Replace all ORM state arguments by the session above */

    var argsWithSession = args.map(function (arg) {
      return isOrmState(arg) ? session : arg;
    });
    /* This is where we call the actual function */

    var result = func.apply(null, argsWithSession); // eslint-disable-line prefer-spread

    /**
     * The metadata for the previous call are no longer valid.
     * Update cached values.
     */

    previous.result = result;
    /* Redux state slice for session.state */

    previous.ormState = ormState;
    /* Rows retrieved by resolved primary key */

    previous.accessedInstances = session.accessedModelInstances;
    /* Foreign key indexes that were used to speed up queries */

    previous.accessedIndexes = session.accessedIndexes;
    /* Tables that had to be scanned completely */

    previous.fullTableScannedModels = session.fullTableScannedModels;
    /* Whether the selector has been called */

    previous.calledBefore = true;
    previous.argsToCompare = argsToCompare;
    return result;
  };
}