"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.QuerySet = void 0;

/**
 * <p>
 * `QuerySet` class is used to build and make queries to the database
 * and operating the resulting set (such as updating attributes
 * or deleting the records).
 * <p>
 *
 * @example queries are built lazily
 * const qs = Book.all()
 *     .filter(book => book.releaseYear > 1999)
 *     .orderBy('name')
 *
 * @description The query is executed only when terminating operations are invoked, such as:
 *
 * - {@link QuerySet#count},
 * - {@link QuerySet#toRefArray}
 * - {@link QuerySet#at} and other indexed reads
 *
 * After the query is executed, the resulting set is cached in the QuerySet instance.
 *
 * QuerySet instances return copies, so chaining filters doesn't
 * mutate the previous instances.
 *
 * @template M type of {@link Model} instances returned by QuerySet's methods.
 * @template InstanceProps additional properties available on QuerySet's elements.
 *
 * @see {@link QuerySet.QueryBuilder}
 */
var QuerySet = /*#__PURE__*/function () {
  return QuerySet;
}();
/**
 * A {@link QuerySet} extended with {@link ManyToMany} specific functionality.
 */


exports.QuerySet = QuerySet;

(function (_QuerySet2) {})(QuerySet || (exports.QuerySet = QuerySet = {}));

var _default = QuerySet;
exports["default"] = _default;